export declare function update<T extends object>(obj: T, updater: (draft: Draft<T>) => void): T;
export declare function clone<T extends object>(obj: T): any;
interface DraftArray<T> {
    readonly length: number;
    [n: number]: Draft<T> | undefined;
    /** Iterator */
    [Symbol.iterator](): IterableIterator<T>;
    /**
     * Returns an iterable of key, value pairs for every entry in the array
     */
    entries(): IterableIterator<[number, T]>;
    /**
     * Returns an iterable of keys in the array
     */
    keys(): IterableIterator<number>;
    /**
     * Returns an iterable of values in the array
     */
    values(): IterableIterator<T>;
    /**
     * Performs the specified action for each element in an array.
     * @param callbackfn  A function that accepts up to three arguments. forEach calls the callbackfn function one time for each element in the array.
     * @param thisArg  An object to which the this keyword can refer in the callbackfn function. If thisArg is omitted, undefined is used as the this value.
     */
    forEach(callbackfn: (value: T, index: number, array: T[]) => void, thisArg?: any): void;
    indexOf(searchElement: T, fromIndex?: number): number;
    includes(searchElement: T, fromIndex?: number): boolean;
    some(predicate: (value: T, index: number, array: T[]) => unknown, thisArg?: any): boolean;
    findIndex(predicate: (value: T, index: number, obj: T[]) => unknown, thisArg?: any): number;
    every(predicate: (value: T, index: number, array: T[]) => unknown, thisArg?: any): boolean;
    map<U>(callbackfn: (value: T, index: number, array: T[]) => U, thisArg?: any): DraftArray<U>;
    filter(predicate: (value: T, index: number, array: T[]) => unknown, thisArg?: any): DraftArray<T>;
    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T): T;
    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue: T): T;
    prepend(item: T): void;
    append(item: T): void;
    insert(item: T, index: number): void;
    insertSorted<U extends string | number>(this: DraftArray<U>, item: U): void;
    insertSorted(item: T, bySortKey: (item: T) => string | number): void;
    updateIf(predicate: (item: T, index: number) => boolean, updateFunction: (item: Draft<T>, index: number) => void): void;
    removeIf(predicate: (item: T, index: number) => boolean): void;
}
interface DraftMap<K, V> extends Omit<Map<K, V>, 'get'> {
    /** Returns an iterable of entries in the map. */
    [Symbol.iterator](): IterableIterator<[K, V]>;
    get(key: K): Draft<V> | undefined;
}
interface DraftSet<E> extends Set<E> {
}
/** Types that should never be drafted */
declare type AtomicObject = Function | Promise<any> | Date | RegExp | Boolean | Number | String;
declare type IsTuple<T extends ReadonlyArray<any>> = T extends never[] ? true : T extends ReadonlyArray<infer U> ? U[] extends T ? false : true : true;
declare type WritableDraft<T> = {
    -readonly [K in keyof T]: Draft<T[K]>;
};
export declare type Draft<T> = T extends AtomicObject ? T : T extends ReadonlyMap<infer K, infer V> ? DraftMap<K, V> : T extends ReadonlySet<infer E> ? DraftSet<E> : T extends ReadonlyArray<infer E> ? IsTuple<T> extends true ? WritableDraft<T> : DraftArray<E> : T extends object ? WritableDraft<T> : T;
export declare function toDraft<T>(array: ReadonlyArray<T>): DraftArray<T>;
export {};
